# -*- coding: utf-8 -*-
"""
Обработчики различных событий

(c)2013 ЗАО "РИПАС" и Демин Федор,
www.ripas.ru dff@ripas.ru
"""
try:
    import gui
    from gui import mainWindow
except:
    gui = None
    mainWindow = None
from PythonQt import QtGui,QtCore
from RoadCentre import kernel,plugins
from settings import userSettings

def selectionChanged():
    if not mainWindow:
        return
    sel=kernel.selection()
    c=mainWindow.findChild(QtGui.QWidget, 'crossView')

    if not len(sel):
        if c: c.setLight(0)
        return
    elif len(sel) == 1:
        o=sel[0]
        if c: c.setLight(o)

def actionConnect(clickConnect):
    from RoadCentre import project
    msg=u'Пользователь инициировал '
    connection=project.connection
#    clickConnect = connection.state <= connection.UnconnectedState
    connection.setProperty('manualDisconnect', not clickConnect)
    if clickConnect:
        msg += u'подключение к серверу '
        fun=connection.start
    else:
        msg += u'отключение от сервера '
        fun=connection.stop
    if gui:
        gui.msgInfo(msg + connection.remoteHost + ':' + str(connection.remotePort))
    fun()

def openFileDialog():
    from RoadCentre import project
    filter=u"Все поддерживаемые форматы ("
    for key in openFileDialog.nameFilter:
        filter=filter+key+" "
    filter=filter[:-1]+")"
    for key in openFileDialog.nameFilter:
        filter=filter+u";;"+openFileDialog.nameFilter[key][0]+u" ("+key+")"
    if not hasattr(openFileDialog,'lastFolder'):
        openFileDialog.lastFolder=project.baseDir
        if len(project.baseDir)==0:
            openFileDialog.lastFolder=QtCore.QDir.homePath()
    fileName = QtGui.QFileDialog.getOpenFileName(mainWindow,u"Открыть файл",openFileDialog.lastFolder,filter)
    if not fileName:
        return
    from os.path import dirname
    openFileDialog.lastFolder=dirname(fileName)
    return openFile(fileName)

def closeProject():
    from RoadCentre import project,kernel
    if project.changed:
        ans=QtGui.QMessageBox.question(mainWindow,u'Вопрос',u'Текущий проект был изменён.\r\nСохранить?',QtGui.QMessageBox.Yes|QtGui.QMessageBox.No|QtGui.QMessageBox.Cancel)
        if ans==QtGui.QMessageBox.Yes:
            saveFile()
        elif ans==QtGui.QMessageBox.Cancel:
            return False
    kernel.itemModel.removeAllSections()
    project.clear()
    if gui:
        gui.osmView.operationWidget.clearLayers()
        if gui.osmView2: gui.osmView2.operationWidget.clearLayers()
        if gui.imageView: gui.imageView.clear()
    project.changed=False
    return True

def openFile(fileName):
    if not fileName: return False
    from os.path import splitext
    from recentFiles import updateRecentList
    try:
        from RoadCentre import project
    except:
        return
    ext = splitext(fileName)[1]
    for key in openFileDialog.nameFilter:
        if splitext(key)[1] == ext:
            if len(openFileDialog.nameFilter[key])>2 and openFileDialog.nameFilter[key][2]:
                if not closeProject():
                    return 0
            res=openFileDialog.nameFilter[key][1](fileName)
            if res:
                updateRecentList(fileName)
            return res

def saveFileDialog():
    from RoadCentre import project
    filter=""
    for key in saveFileDialog.nameFilter:
        if len(filter) > 0:
            filter+=filter+u";;"
        filter+=saveFileDialog.nameFilter[key][0]+u" ("+key+")"
    fileName = QtGui.QFileDialog.getSaveFileName(mainWindow,u"Сохранить файл",project.baseDir,filter)
    if not fileName:
        return None
    result = saveFile(fileName)
    if result:
        return openFile(fileName)
    return result

def saveFile(fileName=None):
    if not fileName:
        from RoadCentre import project
        fileName=project.fileName
        del project
    if not fileName:
        saveFileDialog()
        return
    if gui:
        gui.msgInfo(u"Сохранение проекта в файл %s"%fileName)
    from os.path import splitext
    ext = splitext(fileName)[1]

    result=None
    for key in saveFileDialog.nameFilter:
        if splitext(key)[1] == ext:
            result=saveFileDialog.nameFilter[key][1](fileName)
            break
    if result:
        if gui:
            gui.msgOk(u"Проект сохранён")
        from RoadCentre import project
        project.changed=False
        del project
    else:
        if gui:
            gui.msgError(u"Ошибка при сохранении файла")
    return result

def actionMode(act):
    if not gui:
        return
    mode = int(act.data())
    gui.osmView.dragMode = mode
    if gui.osmView2: gui.osmView2.dragMode = mode
    if gui.imageView: gui.imageView.dragMode = mode

def tileDownload(enable):
    if not gui:
        return
    gui.osmView.enableTileDownload=enable
    userSettings.setValue('enableTileDownload',1 if enable else 0)

def findController(id=0):
    if id==0:
        for sel in kernel.selection():
            if not sel.inherits("Spectr::Light"):
                continue
            id=sel.id
            break
        id=QtGui.QInputDialog.getInt(mainWindow,u'Поиск контроллера',u'Номер:',0)
        if id==0: return
    from RoadCentre import project
    controller=project.object(id)
    if controller==None:
        return
    kernel.selectionModel.clear()
    controller.setSelected(True)
    if gui:
        gui.osmView.zoom=16
        gui.osmView.centerOn(controller.position)

def showControllerOnMap():
    id=0
    for sel in kernel.selection():
        if not sel.inherits("Spectr::Light"):
            continue
        id=sel.id
        break
    if id==0:
        return
    from RoadCentre import project
    controller=project.object(id)
    if controller==None:
        return
    kernel.selectionModel.clear()
    controller.setSelected(True)
    gui.osmView.zoom=16
    gui.osmView.centerOn(controller.position)

def findDetector(obj,dt):
    if not hasattr(plugins,'RoadNetworkPlugin'):
        return
    dt=plugins.RoadNetworkPlugin.findDetector(obj,dt)
    if dt:
        dt.selected=True
        gui.osmView.zoom=21
        gui.osmView.centerOn(dt.position)

def fullScreen(b):
    if not mainWindow:
        return
    if b: mainWindow.showFullScreen()
    else: mainWindow.showNormal()

def setStyle(a):
    QtGui.QApplication.setStyle(a.text)
    #a.checked = True

def AboutQt():
    QtGui.QMessageBox.aboutQt(mainWindow,u"О Qt")

def AboutPlugins():
    if not gui:
        return
    aboutPluginsDlg=gui.widgetFromUi('AboutPlugins.ui')
    if not aboutPluginsDlg:
        return
    pluginsTable=aboutPluginsDlg.findChild('QTableWidget','pluginsTable')
    if not pluginsTable:
        return
    row=0
    for p in kernel.plugins():
        pluginsTable.insertRow(row)
        pluginsTable.setItem(row,0,QtGui.QTableWidgetItem(kernel.objectClassName(p)).clone())
        if hasattr(p,'moduleVersion'):
            pluginsTable.setItem(row,1,QtGui.QTableWidgetItem(p.moduleVersion).clone())
        row+=1
    pluginsTable.resizeColumnsToContents()
    getattr(aboutPluginsDlg,'exec')()

def About():
    if not gui:
        return
    aboutDlg=gui.widgetFromUi('AboutDlg.ui')
    if not aboutDlg:
        return
    label=aboutDlg.findChild('QLabel','label')
    t=label.text.replace('xxxx',QtGui.QApplication.instance().applicationVersion)
    t=t.replace('yyyy',kernel.moduleVersion)
    label.text=t.replace('zzzz',QtGui.QApplication.instance().org_name)
    getattr(aboutDlg,'exec')()

def currentModuleChanged(cur,prev):
    file_path=cur.data(0,QtCore.Qt.UserRole)
    if not file_path:
        AboutExternalModules.textBrowser.enabled=False
        AboutExternalModules.textBrowser.clear()
    else:
        AboutExternalModules.textBrowser.enabled=True
        AboutExternalModules.textBrowser.setText(open(file_path).read())

def processDocFolder(path,root_item):
    import os
    for files in os.walk(path):
        item=QtGui.QTreeWidgetItem((os.path.split(files[0])[1],))
        item_readme=None
        for f in files[2]:
            subitem=QtGui.QTreeWidgetItem((f,))
            file_path=os.path.join(files[0], f)
            if not root_item and not item_readme and 'readme' in f.lower():
                item.setData(0,QtCore.Qt.UserRole,file_path)
            subitem.setData(0,QtCore.Qt.UserRole,file_path)
            item.addChild(subitem.clone())
        for f in files[1]:
            processDocFolder(os.path.join(files[0], f),item)
        if root_item:
            root_item.addChild(item.clone())
        else:
            return item

def AboutExternalModules():
    if not gui:
        return
    dlg=gui.widgetFromUi('ExtModules.ui')
    if not dlg:
        return
    moduleTree=dlg.findChild('QTreeWidget','moduleTree')
    if not moduleTree:
        return
    AboutExternalModules.textBrowser=dlg.findChild('QTextBrowser','textBrowser')
    if not AboutExternalModules.textBrowser:
        return
    moduleTree.connect('currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)',currentModuleChanged)
    items=[]
    import os
    path=os.path.join(os.getcwd(),'../licenses')
    for files in os.walk(path):
        for f in files[1]:
            items.append(processDocFolder(os.path.join(files[0], f),None))
        break
    moduleTree.addTopLevelItems(items)
    getattr(dlg,'exec')()
