# -*- coding: utf-8 -*-
"""
Виджет - таблица выбранных объектов
(c)2011-2012 ЗАО "РИПАС" и Демин Федор,
www.ripas.ru dff@ripas.ru
"""

from PythonQt import QtGui, QtCore, Qt
from RoadCentre import kernel
from gui import mainWindow
class selTab():
    __visibleTypes__ = __builtins__.values()
    def __init__(self, mainWindow, kernel):
        tab = QtGui.QTableWidget()
        self.tab = tab
        self.doc = QtGui.QDockWidget(u'Выбранные объекты', None)
        self.doc.objectName='selTab'
        self.doc.setWidget(tab)
        self.rows = {}
        self.fileName = ''
        mainWindow.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.doc)
        self.selection = kernel.selection
        kernel.selectionModel.connect('selectionChanged(const QItemSelection&, const QItemSelection&)',self.modelSelectionChanged)
        if Qt.qVersion().startswith('5'):
            tab.horizontalHeader().setSectionsMovable(True)
        else:
            tab.horizontalHeader().setMovable(True)
        tab.model().connect('layoutChanged()', self.layoutChanged)
        self.defaultIcon = QtGui.QIcon(QtGui.QPixmap(33,33))

        self.mime=None#QtCore.QMimeData()
        a=QtGui.QAction(u'Выбрать все', tab)
        a.connect('triggered()', tab.selectAll)
        tab.addAction(a)
        a=QtGui.QAction(u'Копировать выбранное в буфер', tab)
        a.connect('triggered()', self.actionCopy)
        tab.addAction(a)
        a=QtGui.QAction(u'Исключить выбранное',tab)
        a.connect('triggered()', self.actionRemove)
        tab.addAction(a)
        a=QtGui.QAction(u'Исключить остальное',tab)
        a.connect('triggered()', self.actionRemoveOther)
        tab.addAction(a)
        a=QtGui.QAction(u'Очистить выделение',tab)
        a.connect('triggered()', kernel.selectionModel.clear)
        tab.addAction(a)
        tab.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
        tab.connect('itemSelectionChanged()',self.updateActions)
        self.updated = False
        self.updateActions()

    def actionCopy(self):
        tab = self.tab
        items = tab.selectedItems()
        rmax = cmax = 0
        rmin = tab.rowCount
        cmin = tab.columnCount

        for i in items:
            row = i.row(); col = tab.visualColumn(i.column())
            rmax = max(rmax, row)
            rmin = min(rmin, row)
            cmax = max(cmax, col)
            cmin = min(cmin, col)
        l = list()
        html='<table border="1" cellpadding="5"><tr>' #'<caption><h2>Selection</caption>'
        text=''
        header = tab.horizontalHeader()
        for i in range(rmin, rmax + 1):
            l.append(['']*(cmax - cmin + 1))
        for i in range(cmin, cmax + 1):
            t = tab.horizontalHeaderItem(header.logicalIndex(i)).text()
            text = text + t + '\t'
            html = html + '<th>' + t + '</th>'
        text = text + '\n'
        html = html + '</tr>'
        for i in items:
            l[i.row() - rmin][tab.visualColumn(i.column()) - cmin] = i.text()
        for i in l:
            html = html + '<tr>'
            for j in i:
                text = text + j + '\t'
                html = html + '<td>' + j + '</td>'
            text = text + '\n'
            html = html + '</tr>'
        html = html + '</table>'
        if not self.mime: self.mime=QtCore.QMimeData()
        self.mime.setHtml(html)
        self.mime.setText(text)
        clipboard=QtGui.QApplication.clipboard()
        clipboard.setMimeData(self.mime)

    def localSelectedObj(self):
        items = self.tab.selectedItems()
        objects = set()
        for i in items:
            for o in self.rows:
                if o in objects: continue
                if i in self.rows[o]:
                    objects.add(o)
                    break
        return objects

    def actionRemove(self):
        obj = self.localSelectedObj()
        self.tab.clearSelection()
        self.updated = True
        for o in obj: o.selected = False
        self.updated = False
        self.modelSelectionChanged()

    def actionRemoveOther(self):
        obj = self.localSelectedObj()
        self.tab.clearSelection()
        self.updated = True
        for o in self.rows:
            if not o in obj: o.selected = False
        self.updated = False
        self.modelSelectionChanged()

    def updateActions(self):
        a = self.tab.actions();
        a[0].setVisible(self.tab.rowCount)
        v = len(self.tab.selectedItems())
        a[1].setVisible(v)
        a[2].setVisible(v)
        a[3].setVisible(v)

    def layoutChanged(self):
        for i in self.rows.items():
            obj = i[0]
            #items = i[1]
            it = self.tab.verticalHeaderItem(i[1][0].row())
            if hasattr(obj,'modelData'): it.setData(QtCore.Qt.DecorationRole, obj.modelData(0,QtCore.Qt.DecorationRole))
            elif hasattr(obj,'picture'): it.setIcon(self.iconFromPicture(obj.picture))
            else: it.setIcon(self.defaultIcon)

    def modelSelectionChanged(self):
        if self.updated: return
        objects=self.selection()
        cnt=self.tab.columnCount
        self.rows = {}
        if len(objects) == 0:
            prop = ()
            if cnt: self.headerSave()
            self.tab.rowCount=0
            self.tab.columnCount=0
            self.headerResore()
            self.updateActions()
            return

        prop = objects[0].__dict__
        lst = list(prop.keys())
        for p in lst:	                                         # закомментируйте эти строки,
            if type(prop[p]) not in selTab.__visibleTypes__: del prop[p] # если хотите видеть в таблице все свойства объекта
        prop = set(prop)
        for obj in objects: prop &= set(obj.__dict__)
        prop = list(prop)
        prop.sort()
        headers=[u'Тип']
        headers.extend(prop)

        self.prop = prop
        self.tab.rowCount=len(objects)
        self.tab.columnCount=len(headers)
        self.tab.setHorizontalHeaderLabels(headers)
        self.tab.sortingEnabled=False
        row=0;
        for obj in objects:
            items = []
            it=QtGui.QTableWidgetItem(kernel.objectClassName(obj))
            self.tab.setItem(row, 0, it)
            items.append(it)
            col=1
            for p in prop:
                it=QtGui.QTableWidgetItem()
                it.setData(QtCore.Qt.DisplayRole, obj.property(p))
                self.tab.setItem(row, col, it)
                items.append(it)
                col+=1

            it=QtGui.QTableWidgetItem()
            if hasattr(obj,'modelData'): it.setData(QtCore.Qt.DecorationRole, obj.modelData(0,QtCore.Qt.DecorationRole))
            elif hasattr(obj,'picture'): it.setIcon(self.iconFromPicture(obj.picture))
            else: it.setIcon(self.defaultIcon)
            self.tab.setVerticalHeaderItem(row,it)
            items.append(it)

            self.rows[obj] = items
            row+=1
        self.tab.sortingEnabled=True
        self.headerResore()
        self.updateActions()

    def objectStateChanged(self, obj):
        if not obj in self.rows: return
        items = self.rows[obj]
        col=1
        for p in self.prop:
            items[col].setData(QtCore.Qt.DisplayRole, obj.property(p))
            col+=1
        it = self.tab.verticalHeaderItem(items[0].row())
        if hasattr(obj,'modelData'): it.setData(QtCore.Qt.DecorationRole, obj.modelData(0,QtCore.Qt.DecorationRole))
        elif hasattr(obj,'picture'): it.setIcon(self.iconFromPicture(obj.picture))

    def headerSave(self):
        h = self.tab.horizontalHeader()
        f=QtCore.QFile(self.stateFileName())
        if not f.open(QtCore.QIODevice.WriteOnly): return False
        f.write(h.saveState())
        f.close()
        return True

    def headerResore(self):
        if not self.tab.columnCount:
            self.fileName = ''
            return False
        fileName = self.stateFileName()
        if self.fileName == fileName: return False
        self.fileName = fileName
        f=QtCore.QFile(fileName)
        if not f.open(QtCore.QIODevice.ReadOnly): return False
        h = self.tab.horizontalHeader()
        h.restoreState(f.readAll())
        f.close()
        return True

    def stateFileName(self):
        d=QtCore.QDir.home()
        app=QtCore.QCoreApplication.instance()

        from sys import platform
        if platform == 'win32': subdir='Application Data/%s/%s/cashe/'%(app.organizationName, app.applicationName)
        else: subdir='.cache/%s/%s/'%(app.organizationName, app.applicationName)
        d.mkpath(subdir)
        d.cd(subdir)

        from hashlib import md5
        m=md5()
        for p in self.prop:  m.update(p.encode('utf8'))
        return d.absoluteFilePath(m.hexdigest())# + '.seltab'

    @staticmethod
    def iconFromPicture(pic):
        px=QtGui.QPixmap(pic.boundingRect().size())
        b=QtGui.QBitmap(px)
        b.clear()
        px.setMask(b)
        pa=QtGui.QPainter()
        pa.begin(px)
        pa.setRenderHint(pa.Antialiasing)
        pa.drawPicture(0,0,pic)
        pa.end()
        i=QtGui.QIcon()
        i.addPixmap(px)
        return i
